@@ -1,3 +1,101 @@
+##A function to compute spectral densities for each dependance
+##models
+specdens <- function(fitted, main, plot = TRUE, ...){
+
+  model <- fitted$model
+  alpha <- fitted$param["alpha"]
+
+  if (model == "log"){
+    ##Logistic case:
+    h <- function(q){
+      if ( (q<=0) || (q>1))
+        return(NaN)
+      else
+        (1/alpha - 1) * (q * (1-q))^(-(1+1/alpha)) *
+          (q^(-1/alpha) + (1-q)^(-1/alpha))^(alpha-2)
+    }
+  }
+
+  if (model == "alog"){
+   ##Asymetric Logistic case:
+    asCoef1 <- fitted$param["asCoef1"]
+    asCoef2 <- fitted$param["asCoef2"]
+    h <- function(q){
+      if ( (q<=0) || (q>1))
+        return(NaN)
+      else
+        (1/alpha - 1) * (asCoef1 * asCoef2)^(1/alpha) *
+          (q * (1-q))^(-(1+1/alpha)) *
+            ((asCoef1/q)^(1/alpha) +
+             (asCoef2/(1-q))^(1/alpha))^(alpha-2)
+    }
+  } 
+
+  if (model == "nlog"){
+    ##Negative Logistic case:
+    h <- function(q){
+      if ( (q<=0) || (q>1))
+        return(NaN)
+      else
+        (1 + alpha) * (q * (1-q))^(alpha-1) *
+          (q^alpha + (1-a)^alpha)^(-1/alpha-2)
+    }
+  }
+
+  if (model == "anlog"){
+    ##Asymetric Negative Logistic case:
+    asCoef1 <- fitted$param["asCoef1"]
+    asCoef2 <- fitted$param["asCoef2"]
+    h <- function(q){
+      if ( (q<=0) || (q>1))
+        return(NaN)
+      else
+        (1 + alpha) * (asCoef1 * asCoef2)^(-alpha) *
+          (q * (1-q))^(alpha-1) *
+            ((q/asCoef1)^alpha +
+             ((1-q)/asCoef2)^alpha)^(-1/alpha-2)
+    }
+  }
+
+  if (model == "mix"){
+    ##Mixed case:
+    h <- function(q){
+      if ( (q<=0) || (q>1))
+        return(NaN)
+      else
+        2 * alpha
+      }
+  }
+
+  if (model == "amix"){
+    ##Mixed case:
+    asCoef <- fitted$param["asCoef"]
+    h <- function(q){
+      if ( (q<=0) || (q>1))
+        return(NaN)
+      else
+        2 * alpha + 6 * asCoef * (1-q)
+    }
+  }
+
+  if (plot){
+    eps <- .Machine$double.eps^0.5
+
+    ##For the logisitic model h is infinite near 0 and 1 thus,
+    if (model == "log")
+      eps <- 0.01
+    
+    if (missing(main))
+      main <- "Spectral Density"
+
+    plot(h, from = eps, to = 1 - eps, main = main, ...)
+  }
+      
+  attributes(h) <- list(model = model)
+  invisible(h)
+}
+
+##A function to plot the Pickands dependance function
 pickdep <- function(fitted, main, bound = TRUE, plot = TRUE,
                     ...){
 
@@ -90,9 +188,9 @@ pickdep <- function(fitted, main, bound = TRUE, plot = TRUE,
   invisible(A)
 }
 
-
-bvretlev <- function(fitted, p = seq(0.75,0.95,0.05), main,
-                     n = 5000, only.excess = FALSE, ...){
+##Bivariate return level plot
+retlev.bvpot <- function(fitted, p = seq(0.75,0.95,0.05), main,
+                         n = 5000, only.excess = FALSE, ...){
   
   if (all(class(fitted) != "bvpot"))
     stop("``fitted'' should be an object of class ``bvpot''.")
@@ -135,7 +233,7 @@ bvretlev <- function(fitted, p = seq(0.75,0.95,0.05), main,
   
   plot(data1, data2, main = main, ...)
   lines(y1, y2)
-
+  
   ##Add the marginal threshold axis
   abline(v = fitted$threshold[1])
   abline(h = fitted$threshold[2])
@@ -144,26 +242,28 @@ bvretlev <- function(fitted, p = seq(0.75,0.95,0.05), main,
 
 }
 
-plot.bvpot <- function(x, mains, which = 1:2,
+##The generic function for graphical diagnostic of a bivariate
+##pot object
+plot.bvpot <- function(x, mains, which = 1:3,
                        ask = nb.fig < length(which) &&
                        dev.interactive(), ...){
 
-  if (!is.numeric(which) || any(which < 1) || any(which > 2)) 
-        stop("`which' must be in 1:2")
+  if (!is.numeric(which) || any(which < 1) || any(which > 3)) 
+        stop("`which' must be in 1:3")
 
   if(missing(mains))
     mains <- c("Pickands' Dependence Function",
-               "Bivariate Return Level Plot")
+               "Bivariate Return Level Plot", "Spectral Density")
 
   else
-    if (length(mains) != 2){
+    if (length(mains) != 3){
       warning("``mains'' must be of length two. Passing to default titles.")
       mains <- c("Pickands' Dependence Function",
-                 "Bivariate Return Level Plot")
+                 "Bivariate Return Level Plot", "Spectral Density")
     }
 
 
-  show <- rep(FALSE, 2)
+  show <- rep(FALSE, 3)
   show[which] <- TRUE
   nb.fig <- prod(par("mfcol"))
   
@@ -176,6 +276,9 @@ plot.bvpot <- function(x, mains, which = 1:2,
     pickdep(x, main = mains[1], ...)
 
   if (show[2])
-    bvretlev(x, main = mains[2], ...)
+    retlev(x, main = mains[2], ...)
+
+  if (show[3])
+    specdens(x, main = mains[3], ...)
 
 }
